import { _RouteRecordBase } from 'vue-router';
import { RawRouteComponent, NavigationGuardReturn } from '../vue-router-utils';
import { RoutiderPath, RoutiderPaths } from '../options/path';
import { RoutiderLocation, RoutiderLocationN } from './location';
import { RoutiderNavigationGuardNext } from '../router/navigationGuard';
import { RouteRecordName } from '../options/name';
/**
 * Typed _RouteRecordProps
 */
export declare type _RouteRecordProps<Params extends string | undefined, Queries extends string | undefined, RouteName extends RouteRecordName = RouteRecordName> = boolean | Record<string, any> | ((to: RoutiderLocation<Params, undefined, Queries, RouteName>) => Record<string, any>);
declare type RoutiderRouteRecordRedirectOption<Params extends string | undefined, Queries extends string | undefined, RouteName extends RouteRecordName = RouteRecordName> = (to: RoutiderLocation<Params, undefined, Queries, RouteName>) => unknown;
/**
 * Typed `NavigationGuardWithThis` for beforeEnter
 */
export interface RoutiderBeforeEnterGuardWithThis<T, Params extends string | undefined, Queries extends string | undefined, RouteName extends RouteRecordName = RouteRecordName> {
    (this: T, to: RoutiderLocationN<Params, undefined, Queries, RouteName>, from: RoutiderLocationN<undefined, undefined, Queries, RouteName>, next: RoutiderNavigationGuardNext<Record<never, never>>): NavigationGuardReturn | Promise<NavigationGuardReturn>;
}
/**
 * Typed RouteRecordBase
 */
export interface _RoutiderRouteRecordBase<Params extends string | undefined, Queries extends string | undefined> extends Omit<_RouteRecordBase, 'path' | 'query' | 'beforeEnter'> {
    /**
     * `name` will be automatically set from key
     */
    name?: never;
    /**
     * Path of the record. Should start with / unless the record is the child of another record.
     * @example createPath`/users/${'id'}` matches `/users/1` as well as `/users/posva`. '/users' matches only `/users`.
     */
    path: RoutiderPath<Params> | string | RoutiderPaths<Params> | string[];
    /**
     * Use `path` with an array for aliases.
     */
    alias?: never;
    /**
     * Define queries which may receive
     */
    query?: Queries[];
    /**
     * Nested routes are not supported by vue-routider because of type inference limitations.
     * @see {@link https://github.com/sapphi-red/vue-routider/issues/4}
     */
    children?: never;
    redirect?: RoutiderRouteRecordRedirectOption<Params, Queries>;
    beforeEnter?: RoutiderBeforeEnterGuardWithThis<undefined, Params, Queries> | RoutiderBeforeEnterGuardWithThis<undefined, Params, Queries>[];
}
interface RoutiderRouteRecordSingleView<Params extends string | undefined, Queries extends string | undefined> extends _RoutiderRouteRecordBase<Params, Queries> {
    /**
     * Component to display when the URL matches this route.
     */
    component: RawRouteComponent;
    /**
     * Allow passing down params as props to the component rendered by `router-view`.
     */
    props?: _RouteRecordProps<Params, Queries>;
}
interface RoutiderRouteRecordMultipleViews<Params extends string | undefined, Queries extends string | undefined> extends _RoutiderRouteRecordBase<Params, Queries> {
    /**
     * Components to display when the URL matches this route. Allow using named views.
     */
    components: Record<string, RawRouteComponent>;
    /**
     * Allow passing down params as props to the component rendered by
     * `router-view`. Should be an object with the same keys as `components` or a
     * boolean to be applied to every component.
     */
    props?: Record<string, _RouteRecordProps<Params, Queries>> | boolean;
}
interface RoutiderRouteRecordRedirect<Params extends string | undefined, Queries extends string | undefined> extends _RoutiderRouteRecordBase<Params, Queries> {
    redirect: RoutiderRouteRecordRedirectOption<Params, Queries>;
    component?: never;
    components?: never;
    children?: never;
}
export declare type RoutiderRouteRecord<Params extends string | undefined = string | undefined, Queries extends string | undefined = string | undefined> = RoutiderRouteRecordSingleView<Params, Queries> | RoutiderRouteRecordMultipleViews<Params, Queries> | RoutiderRouteRecordRedirect<Params, Queries>;
interface PathAndAlias {
    path: string;
    alias?: string[];
}
export declare const pathToPathAndAlias: (path: RoutiderPath | string | RoutiderPaths | string[]) => PathAndAlias;
declare type ConvertToUndefinedIfStringPath<Route extends RoutiderRouteRecord> = Route extends RoutiderRouteRecord<infer P, infer Q> ? string extends P ? RoutiderRouteRecord<undefined, Q> : Route : never;
declare type ConvertToUndefinedIfStringQuery<Route extends RoutiderRouteRecord> = Route extends RoutiderRouteRecord<infer P, infer Q> ? string extends Q ? RoutiderRouteRecord<P, undefined> : Route : never;
export declare const createRoute: <Params extends string | undefined, Queries extends string | undefined>(route: RoutiderRouteRecord<Params, Queries>) => ConvertToUndefinedIfStringQuery<string extends Params ? RoutiderRouteRecord<undefined, Queries> : RoutiderRouteRecordSingleView<Params, Queries>> | ConvertToUndefinedIfStringQuery<string extends Params ? RoutiderRouteRecord<undefined, Queries> : RoutiderRouteRecordMultipleViews<Params, Queries>> | ConvertToUndefinedIfStringQuery<string extends Params ? RoutiderRouteRecord<undefined, Queries> : RoutiderRouteRecordRedirect<Params, Queries>>;
export {};
