'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var warning = _interopDefault(require('tiny-warning'));
var vueRouter = require('vue-router');

function createPath(literals, ...placeholders) {
  let path = '';

  for (let i = 0; i < placeholders.length; i++) {
    const l = literals[i];
    const p = placeholders[i];
    path += `${l}:${p}`;
     warning(!l.includes(':'), `vue-routider: \`:\` should not be included in createPath argument. (${l})`) ;
     warning(!p.includes(':'), `vue-routider: \`:\` should not be included in createPath argument. (${p})`) ;
  }

  const lastL = literals[literals.length - 1];
  path += lastL;
   warning(!lastL.includes(':'), `vue-routider: \`:\` should not be included in createPath argument. (${lastL})`) ;
  return path;
}
function createPaths(...args) {
  return args;
}
const pathToString = path => path; // eslint-disable-next-line @typescript-eslint/no-explicit-any

const pathsToString = paths => paths;

// Code based on vue-router
const pathToPathAndAlias = path => {
  if (!Array.isArray(path)) return {
    path: pathToString(path)
  };

  if (path.length <= 0) {
    throw new Error('Path property in Route of RoutiderOptions must have at least 1 path.');
  }

  const pathStrings = pathsToString(path); // eslint-disable-next-line @typescript-eslint/no-non-null-assertion

  const pathProp = pathStrings.shift();
  return {
    path: pathProp,
    alias: pathStrings
  };
};
const createRoute = route => route;

const routiderOptionsToRouterOptions = options => {
  const routes = Object.entries(options.routes).map(([name, route]) => ({ ...route,
    name,
    ...pathToPathAndAlias(route.path)
  }));
  return { ...options,
    routes
  };
};

const createRoutiderRouter = router => {
  const isRouteName = (location, name) => name === location.name;

  const getOptionalTypedRoute = location => location;

  return { ...router,
    isRouteName,
    getOptionalTypedRoute
  };
};

const warnIfIncorrectRoute = (route, name) => {
  var _route$matched$;

  if (name === null) return;
  const matchedName = (_route$matched$ = route.matched[0]) === null || _route$matched$ === void 0 ? void 0 : _route$matched$.name;
  if (matchedName === undefined) return;
  const names = Array.isArray(name) ? name : [name];
   warning(names.includes(matchedName), `vue-routider: Incorrect useRoute usage. Path is now matched '${String(matchedName)}' but \`useRoute([${names.join(',')}])\` is used.`) ;
};

const createRoutider = options => {
  const routerOptions = routiderOptionsToRouterOptions(options);
  const rawRouter = vueRouter.createRouter(routerOptions);
  const router = createRoutiderRouter(rawRouter);

  const useRouter = () => router;

  const useRoute = name => {
    const route = vueRouter.useRoute();

    {
      warnIfIncorrectRoute(route, name);
    }

    return route;
  };

  const onBeforeRouteLeave = leaveGuard => {
    vueRouter.onBeforeRouteLeave(leaveGuard);
  };

  const onBeforeRouteUpdate = updateGuard => {
    vueRouter.onBeforeRouteUpdate(updateGuard);
  };

  const ensureLocationType = location => location;

  return {
    rawRouter,
    router,
    useRouter,
    useRoute,
    onBeforeRouteLeave,
    onBeforeRouteUpdate,
    ensureLocationType
  };
};

exports.createPath = createPath;
exports.createPaths = createPaths;
exports.createRoute = createRoute;
exports.createRoutider = createRoutider;
exports.createRoutiderRouter = createRoutiderRouter;
exports.pathToPathAndAlias = pathToPathAndAlias;
exports.pathToString = pathToString;
exports.pathsToString = pathsToString;
exports.routiderOptionsToRouterOptions = routiderOptionsToRouterOptions;
//# sourceMappingURL=vue-routider.cjs.development.js.map
