import warning from 'tiny-warning';
import { createRouter, useRoute, onBeforeRouteLeave, onBeforeRouteUpdate } from 'vue-router';

function createPath(literals, ...placeholders) {
  let path = '';

  for (let i = 0; i < placeholders.length; i++) {
    const l = literals[i];
    const p = placeholders[i];
    path += `${l}:${p}`;
    process.env.NODE_ENV !== "production" ? warning(!l.includes(':'), `vue-routider: \`:\` should not be included in createPath argument. (${l})`) : void 0;
    process.env.NODE_ENV !== "production" ? warning(!p.includes(':'), `vue-routider: \`:\` should not be included in createPath argument. (${p})`) : void 0;
  }

  const lastL = literals[literals.length - 1];
  path += lastL;
  process.env.NODE_ENV !== "production" ? warning(!lastL.includes(':'), `vue-routider: \`:\` should not be included in createPath argument. (${lastL})`) : void 0;
  return path;
}
function createPaths(...args) {
  return args;
}
const pathToString = path => path; // eslint-disable-next-line @typescript-eslint/no-explicit-any

const pathsToString = paths => paths;

// Code based on vue-router
const pathToPathAndAlias = path => {
  if (!Array.isArray(path)) return {
    path: pathToString(path)
  };

  if (path.length <= 0) {
    throw new Error('Path property in Route of RoutiderOptions must have at least 1 path.');
  }

  const pathStrings = pathsToString(path); // eslint-disable-next-line @typescript-eslint/no-non-null-assertion

  const pathProp = pathStrings.shift();
  return {
    path: pathProp,
    alias: pathStrings
  };
};
const createRoute = route => route;

const routiderOptionsToRouterOptions = options => {
  const routes = Object.entries(options.routes).map(([name, route]) => ({ ...route,
    name,
    ...pathToPathAndAlias(route.path)
  }));
  return { ...options,
    routes
  };
};

const createRoutiderRouter = router => {
  const isRouteName = (location, name) => name === location.name;

  const getOptionalTypedRoute = location => location;

  return { ...router,
    isRouteName,
    getOptionalTypedRoute
  };
};

const warnIfIncorrectRoute = (route, name) => {
  var _route$matched$;

  if (name === null) return;
  const matchedName = (_route$matched$ = route.matched[0]) === null || _route$matched$ === void 0 ? void 0 : _route$matched$.name;
  if (matchedName === undefined) return;
  const names = Array.isArray(name) ? name : [name];
  process.env.NODE_ENV !== "production" ? warning(names.includes(matchedName), `vue-routider: Incorrect useRoute usage. Path is now matched '${String(matchedName)}' but \`useRoute([${names.join(',')}])\` is used.`) : void 0;
};

const createRoutider = options => {
  const routerOptions = routiderOptionsToRouterOptions(options);
  const rawRouter = createRouter(routerOptions);
  const router = createRoutiderRouter(rawRouter);

  const useRouter = () => router;

  const useRoute$1 = name => {
    const route = useRoute();

    if (process.env.NODE_ENV !== "production") {
      warnIfIncorrectRoute(route, name);
    }

    return route;
  };

  const onBeforeRouteLeave$1 = leaveGuard => {
    onBeforeRouteLeave(leaveGuard);
  };

  const onBeforeRouteUpdate$1 = updateGuard => {
    onBeforeRouteUpdate(updateGuard);
  };

  const ensureLocationType = location => location;

  return {
    rawRouter,
    router,
    useRouter,
    useRoute: useRoute$1,
    onBeforeRouteLeave: onBeforeRouteLeave$1,
    onBeforeRouteUpdate: onBeforeRouteUpdate$1,
    ensureLocationType
  };
};

export { createPath, createPaths, createRoute, createRoutider, createRoutiderRouter, pathToPathAndAlias, pathToString, pathsToString, routiderOptionsToRouterOptions };
//# sourceMappingURL=vue-routider.esm.js.map
